import OME.*;
import java.awt.Point;
import java.awt.Image;
import javax.swing.ImageIcon;
import java.util.Iterator;
import java.util.Collection;
import java.util.*;
import java.io.File;

/** The IStar plugin.  This plugin will perform IStar analyses, and add
 *  modelling shortcuts to the toolbar.
 */

public class IStarPlugin implements OMEPlugin {
    
    // the names of the I-Star links
    static String[] linknames = {"Dependency link", "Decomposition link",
			  "Contribution link", "Means-ends link",
			  "ISA", "INS", "Plays","Occupies", "Covers", 
			  "Is-Part-of", "Prior-to", "T_OR",
			  "T_And", "Failure", "Shared"};	

    static String[] elementnames = {"Actor", "Agent", "Role", "Position",
			"Resource", "Task", "Goal", "Softgoal",
			"use-case-starting-point", "use-case-ending-point"};

    OMEModel model;
    private boolean doallplausibleanalysis = true;
    private boolean dodependencyanalysis = false;
    private boolean dorationaleanalysis = false;
    private static String IMAGE_DIR = "images";
    
    public IStarPlugin(OMEModel m) {
	model = m;
    }

    /** Tests for presence of basic IStar type.*/
    static public boolean isCompatibleWith(OMEModel model) {
	return (model.getFramework().getType("Position") != null);
    }
    
    /** Returns a collection  of our <code>Method</code>s that are to be
     *  placed on the OME toolbar. */
    public Collection getToolbarMethods(View view) {

	Vector v = new Vector();
	
	// Need to be able to create i* links and elements.
	MenuMethod em = new MenuMethod("Create Istar Element");
	em.setSubmenu(getElementMethods(view));
	v.add(em);
	
	MenuMethod lm = new MenuMethod("Create Istar Link");
	lm.setSubmenu(getLinkMethods(view));
	v.add(lm);
	
	// We need to find the dependency link type, and the resource type.
	Object dltype = model.getFramework().getType("Dependency link");
	Object rtype = model.getFramework().getType("Resource");
	Object pltype = model.getFramework().getType("Prior-to");
	Object stype = model.getFramework().getType("use-case-starting-point");
	Object etype = model.getFramework().getType("use-case-ending-point");
	
	// Now we can create the Resource-dependency shortcut.
	PluginMethod m = new CreateDependency(view, rtype, dltype);
	PluginMethod m1 = new CreateUseCasePath(view, stype, etype, pltype);
	MenuMethod mm = new MenuMethod("Create Dependency Link");
	mm.addItem(m);
	mm.addItem(m1);
	
	v.add(mm);
	return v;
    }

    /** Returns a collection  of our <code>Method</code>s that are to be
     *  placed on the OME menubar. 
     */
    public Collection getMenubarMethods (View v) {
	MenuMethod istaroptions = new MenuMethod("I* Options");
	istaroptions.addItem(new allPlausibleAnalysis());
	istaroptions.addItem(new dependencyAnalysis());
	istaroptions.addItem(new rationaleAnalysis());
    
	return Collections.singleton(istaroptions);
    }

    /** Returns a collection  of our <code>Method</code>s that are to be
     *  placed in the OME popup-menu (when the user clicks the right mouse
     *  button). 
     */
    public Collection getPopupMethods(View view) {
	D.o("Getting i* popup methods");
	LinkedList ll = new LinkedList();
	
	MenuMethod em = new MenuMethod("Create Istar Element");
	em.setSubmenu(getElementMethods(view));
	
	MenuMethod lm = new MenuMethod("Create Istar Link");
	lm.setSubmenu(getLinkMethods(view));
	
	ll.add(new PopupMenuSeparatorMethod(view));
	ll.add(em);
	ll.add(lm);
	return ll;
    }


    /** Return a collection of element creation methods for the i* element
      * types. */
    private Collection getElementMethods(View view) {
	Vector elementmethods = new Vector(elementnames.length);
	for (int i=0; i<elementnames.length; i++) {
	    Object type = model.getFramework().getType(elementnames[i]);
	    PluginMethod method = new
		CreateElementMethod(type,elementnames[i],view);
	    elementmethods.add(method);
	}
	return elementmethods;
    }
    
    /** Return a collection of link creation methods for the i* link types. */
    private Collection getLinkMethods(View view) {
	Vector linkmethods = new Vector(linknames.length);
	for (int i=0; i<linknames.length; i++) {
	    Object type = model.getFramework().getType(linknames[i]);
	    PluginMethod method = new
		CreateLinkMethod(type,linknames[i],view);
	    linkmethods.add(method);
	}
	return linkmethods;
    }

    private void setDoAllPlausibleAnalysis(boolean b){
	doallplausibleanalysis = b;
    }

    private boolean shouldDoAllPlausibleAnalysis(){
	return doallplausibleanalysis;
    }
    private void setDoDependencyAnalysis(boolean b){
	dodependencyanalysis = b;
    }

    private boolean shouldDoDependencyAnalysis(){
	return dodependencyanalysis;
    }

    private void setDoRationaleAnalysis(boolean b){
	dorationaleanalysis = b;
    }
    
    private boolean shouldDoRationaleAnalysis(){
	return dorationaleanalysis;
    }
    
    private class allPlausibleAnalysis extends AbstractPluginMethod{

	private Image isonimage;

	public allPlausibleAnalysis(){
	    ImageIcon ii = new
		ImageIcon(IMAGE_DIR+File.separatorChar+"check.gif");
	    isonimage = ii.getImage();
	}

	public String getName(){
	    return "Auto do All Plausible Analysis";
	}

	public Image getImage(){
	    if (shouldDoAllPlausibleAnalysis()){
		return isonimage;
	    } else {
		return null;
	    }
	}

	public void invoke(){
	    setDoAllPlausibleAnalysis(!shouldDoAllPlausibleAnalysis());
	    if (shouldDoAllPlausibleAnalysis()){
	    setDoDependencyAnalysis(false);
	    setDoRationaleAnalysis(false);
	    }
	}
    }
 
    private class dependencyAnalysis extends AbstractPluginMethod{

	private Image isonimage;

	public dependencyAnalysis(){
	    ImageIcon ii = new
		ImageIcon(IMAGE_DIR+File.separatorChar+"check.gif");
	    isonimage = ii.getImage();
	}

	public String getName(){
	    return "Inter-agent Dependency Analysis";
	}

	public Image getImage(){
	    if (shouldDoDependencyAnalysis()){
		return isonimage;
	    } else {
		return null;
	    }
	}

	public void invoke(){
	    setDoDependencyAnalysis(!shouldDoDependencyAnalysis());
	    if (shouldDoDependencyAnalysis()){
		setDoAllPlausibleAnalysis(false);
	    }
	}
    }
    
      private class rationaleAnalysis extends AbstractPluginMethod{

	private Image isonimage;

	public rationaleAnalysis(){
	    ImageIcon ii = new
		ImageIcon(IMAGE_DIR+File.separatorChar+"check.gif");
	    isonimage = ii.getImage();
	}

	public String getName(){
	    return "Agent Rationales Analysis";
	}

	public Image getImage(){
	    if (shouldDoRationaleAnalysis()){
		return isonimage;
	    } else {
		return null;
	    }
	}

	public void invoke(){
	    setDoRationaleAnalysis(!shouldDoRationaleAnalysis());
	    if (shouldDoRationaleAnalysis()){
		setDoAllPlausibleAnalysis(false);
	    }
	}
    }
  
    
    ////////////////////////////////////////////////////////////////////////
    //									  //
    //	    METHODS							  //
    //									  //
    ////////////////////////////////////////////////////////////////////////

    private class CreateDependency extends AbstractPluginMethod {

	View view;
	Object dependumType;
	Object linkType;
	Image image;
	ViewElement depender;
	ViewElement dependee;
	PluginParameter nextp;
	
	public CreateDependency(View v, Object dependumType, Object linkType) {
	    view = v;
	    this.dependumType = dependumType;
	    this.linkType = linkType;

	    nextp = sourceParameter();
	}

	public String getName() {
	    return model.getFramework().getName(dependumType)+" dependency";
	}

	public Image getImage() {
	    ImageIcon ic = new ImageIcon("images/link-el-link.gif");
	    return ic.getImage();
	}

	public Collection getSubmenu() {
	    return null;
	}
	
	public PluginParameter nextParameter() {
	    return nextp;
	}

	public void passParameter(Collection c) {	
	    if (!c.iterator().hasNext()) {
		// There is nothing there.  We weren't expecting that.
	    } else if (depender == null) {
		
		// We have received the source object.
		depender = (ViewElement)c.iterator().next();

		// And we now want the destination object.
		nextp = destinationParameter();
		
	    } else {
	    
		// We have received the destination object.
		dependee = (ViewElement)c.iterator().next();

		// And we are done getting parameters
		nextp = null;
	    }
	}

	public void invoke() {

	    // I am tired when I write this.

	    View.Location froml = depender.getLocation();
	    View.Location tol = dependee.getLocation();
	    ViewObject newelement = null;
	    ViewObject sourcelink = null;
	    ViewObject destlink = null;
	    
	    try {
		View.GraphicLocation fromgl = (View.GraphicLocation)froml;
		View.GraphicLocation togl = (View.GraphicLocation)tol;
		View.GraphicLocation newgl = new View.GraphicLocation();
		Point p = new Point(
			(int)(togl.getPoint().getX() + 
			      fromgl.getPoint().getX())/2,
			(int)(togl.getPoint().getY() + 
			      fromgl.getPoint().getY())/2);
		newgl.setPoint(p);

		// And create the guys involved.
		newelement = view.createElement(dependumType, newgl);
		sourcelink = view.createLink(linkType,
			depender.getModelObject(), newelement.getModelObject());
		destlink = view.createLink(linkType,
			newelement.getModelObject(), dependee.getModelObject());
		view.promptRename(newelement);
	    } catch (Exception e) {
		// Do something!!!
		D.o(e);
	    }
	    
	    resetMethod();
	}
	
	public void cancelled() {
	    resetMethod();
	}

	private void resetMethod() {
	    nextp = sourceParameter();
	    depender = null;
	    dependee = null;
	}

	private PluginParameter sourceParameter() {
	    // We want a ViewObject for one end of the link.
	    Class ptype = null;
	    try {
		ptype = Class.forName("OME.OMEElement");  // s/b ViewElement
	    } catch (Exception e) {
		D.a(e);
	    }
	    return new PluginParameter(PluginParameter.CARDINALITY_ONE,
		    "Select the DEPENDER of the " + dependumType +
		    ".", ptype);
	}

	private PluginParameter destinationParameter() {
	    // We want a ViewObject for the other end of the link.
	    Class ptype = null;
	    try {
		ptype = Class.forName("OME.OMEElement");
	    } catch (Exception e) {
		D.a(e);
	    }
	    return new PluginParameter(PluginParameter.CARDINALITY_ONE,
		    "Select the DEPENDEE of the " + dependumType +
		    ".", ptype);
	}

    }

    ///////////////////////////////////////////////////////////
    //
    // This is a method to create use case path.
    //
    //
    /////////////////////////////////////////////////////////
    
      private class CreateUseCasePath extends AbstractPluginMethod {

	View view;
	Image image;
	Object stype, etype, pltype;
	View.Location startpoint;
	View.Location endpoint;
	PluginParameter nextp;

	public CreateUseCasePath(View v, Object stype, Object etype, Object
		pltype) {
	    view = v;
	    this.stype = stype;
	    this.etype = etype;
	    this.pltype = pltype;
	    nextp = sourceParameter();
	}

	public String getName() {
	    return "Use Case Path";
	}

	public Image getImage() {
	    ImageIcon ic = new ImageIcon("images/use-case-path.gif");
	    return ic.getImage();
	}

	public Collection getSubmenu() {
	    return null;
	}
	
	public PluginParameter nextParameter() {
	    return nextp;
	}

	public void passParameter(Collection c) {	
	
	    if (!c.iterator().hasNext()) {
	    // There is nothing there.  We weren't expecting that.
	
	    } else if (startpoint == null) {
		
//		ViewContext context = (ViewContext)c.iterator().next();
//		if (context.associatedLocation() != null) {
//		    startpoint = context.associatedLocation();
		startpoint = (View.Location)c.iterator().next();
		nextp = destinationParameter();
//		}
	    
	    }else if (endpoint == null) {
		// We have received the destination object.
		endpoint = (View.Location)c.iterator().next();

		// And we are done getting parameters
		nextp = null;
	
	    } else {
	    // We should never get here.
		D.a("Invalid state in Create Use Case Path.");
	    }
    	}

	public void invoke() {

	    // I am tired when I write this.

	    ViewObject newstart = null;
	    ViewObject newend = null;
	    ViewObject priortolink = null;
	    
	    try {

		// And create the guys involved.
		//newelement = view.createElement(dependumType, newgl);
		newstart = view.createElement(stype, startpoint);
		view.promptRename(newstart);
		newend = view.createElement(etype, endpoint);
		view.promptRename(newend);
		priortolink = view.createLink(pltype,
			newstart.getModelObject(),newend.getModelObject());
	    } catch (Exception e) {
		// Do something!!!
		D.o(e);
	    }
	    
	    resetMethod();
	}
	
	public void cancelled() {
	    resetMethod();
	}

	private void resetMethod() {
	    nextp = sourceParameter();
	    startpoint = null;
	    endpoint = null;
	}

	private PluginParameter sourceParameter() {
	    // We want a ViewObject for one end of the link.
	return new PluginParameter(PluginParameter.CARDINALITY_ONE,
		"Please select where to put the new use case path starting point.", 
		PluginParameter.LocationType);
	}

	private PluginParameter destinationParameter() {
	    // We want a ViewObject for the other end of the link.
	    return new PluginParameter(PluginParameter.CARDINALITY_ONE,
		"Please select where to put the new use case path ending point.", 
		PluginParameter.LocationType);	
	}
    }
  
}
